@@ -6,9 +6,7 @@ use holochain_wasmer_guest::*;
 /// This is the root of the [ `Path` ] tree.
 ///
 /// Forms the entry point to all anchors so that agents can navigate down the tree from here.
-///
-/// The string "hdkanchor".
-pub const ROOT: &str = "hdkanchor";
+pub const ROOT: &[u8; 2] = &[0x00, 0x00];
 
 #[derive(PartialEq, SerializedBytes, serde::Serialize, serde::Deserialize, Debug, Clone)]
 /// An anchor can only be 1 or 2 levels deep as "type" and "text".
@@ -27,19 +25,20 @@ pub struct Anchor {
 }
 
 // Provide all the default entry conventions for anchors.
-entry_def!(Anchor Path::entry_def());
+entry_def!(Anchor PathEntry::entry_def());
 
 /// Anchors are just a special case of path, so we can move from anchor to path losslessly.
 /// We simply format the anchor structure into a string that works with the path string handling.
 impl From<&Anchor> for Path {
     fn from(anchor: &Anchor) -> Self {
-        Self::from(&format!(
-            "{1}{0}{2}{0}{3}",
-            crate::hash_path::path::DELIMITER,
-            ROOT,
-            anchor.anchor_type,
-            anchor.anchor_text.as_ref().unwrap_or(&String::default())
-        ))
+        let mut components = vec![
+            Component::new(ROOT.to_vec()),
+            Component::from(anchor.anchor_type.as_bytes().to_vec()),
+        ];
+        if let Some(text) = anchor.anchor_text.as_ref() {
+            components.push(Component::from(text.as_bytes().to_vec()));
+        }
+        components.into()
     }
 }
 
@@ -47,31 +46,38 @@ impl From<&Anchor> for Path {
 /// The obvious example would be a path of binary data that is not valid utf-8 strings or a path
 /// that is more than 2 levels deep.
 impl TryFrom<&Path> for Anchor {
-    type Error = SerializedBytesError;
+    type Error = WasmError;
     fn try_from(path: &Path) -> Result<Self, Self::Error> {
         let components: Vec<Component> = path.as_ref().to_owned();
         if components.len() == 2 || components.len() == 3 {
-            if components[0] == Component::from(ROOT) {
+            if components[0] == Component::new(ROOT.to_vec()) {
                 Ok(Anchor {
-                    anchor_type: (&components[1]).try_into()?,
+                    anchor_type: std::str::from_utf8(components[1].as_ref())
+                        .map_err(|e| SerializedBytesError::Deserialize(e.to_string()))?
+                        .to_string(),
                     anchor_text: {
                         match components.get(2) {
-                            Some(component) => Some(component.try_into()?),
+                            Some(component) => Some(
+                                std::str::from_utf8(component.as_ref())
+                                    .map_err(|e| SerializedBytesError::Deserialize(e.to_string()))?
+                                    .to_string(),
+                            ),
                             None => None,
                         }
                     },
                 })
             } else {
-                Err(SerializedBytesError::Deserialize(format!(
-                    "Bad anchor path root {:0?} should be {:1?}",
-                    components[0].as_ref(),
-                    ROOT.as_bytes(),
+                Err(WasmError::Serialize(SerializedBytesError::Deserialize(
+                    format!(
+                        "Bad anchor path root {:0?} should be {:1?}",
+                        components[0].as_ref(),
+                        ROOT,
+                    ),
                 )))
             }
         } else {
-            Err(SerializedBytesError::Deserialize(format!(
-                "Bad anchor path length {}",
-                components.len()
+            Err(WasmError::Serialize(SerializedBytesError::Deserialize(
+                format!("Bad anchor path length {}", components.len()),
             )))
         }
     }
@@ -86,28 +92,13 @@ pub fn anchor(anchor_type: String, anchor_text: String) -> ExternResult<holo_has
     })
         .into();
     path.ensure()?;
-    path.hash()
-}
-
-/// Attempt to get an anchor by its hash.
-/// Returns None if the hash doesn't point to an anchor.
-/// We can't do anything fancy like ensure the anchor if not exists because we only have a hash.
-pub fn get_anchor(anchor_address: EntryHash) -> ExternResult<Option<Anchor>> {
-    Ok(
-        match crate::prelude::get(anchor_address, GetOptions::content())?.and_then(|el| el.into()) {
-            Some(Entry::App(eb)) => {
-                let path = Path::try_from(SerializedBytes::from(eb))?;
-                Some(Anchor::try_from(&path)?)
-            }
-            _ => None,
-        },
-    )
+    path.path_entry_hash()
 }
 
 /// Returns every entry hash in a vector from the root of an anchor.
 /// Hashes are sorted in the same way that paths sort children.
 pub fn list_anchor_type_addresses() -> ExternResult<Vec<EntryHash>> {
-    let links = Path::from(ROOT)
+    let links = Path::from(vec![Component::new(ROOT.to_vec())])
         .children()?
         .into_iter()
         .map(|link| link.target)
@@ -124,7 +115,6 @@ pub fn list_anchor_addresses(anchor_type: String) -> ExternResult<Vec<EntryHash>
         anchor_text: None,
     })
         .into();
-    path.ensure()?;
     let links = path
         .children()?
         .into_iter()
@@ -144,18 +134,15 @@ pub fn list_anchor_tags(anchor_type: String) -> ExternResult<Vec<String>> {
     })
         .into();
     path.ensure()?;
-    let hopefully_anchor_tags: Result<Vec<String>, SerializedBytesError> = path
-        .children()?
+    let hopefully_anchor_tags: Result<Vec<String>, WasmError> = path
+        .children_paths()?
         .into_iter()
-        .map(|link| match Path::try_from(&link.tag) {
-            Ok(path) => match Anchor::try_from(&path) {
-                Ok(anchor) => match anchor.anchor_text {
-                    Some(text) => Ok(text),
-                    None => Err(SerializedBytesError::Deserialize(
-                        "missing anchor text".into(),
-                    )),
-                },
-                Err(e) => Err(e),
+        .map(|path| match Anchor::try_from(&path) {
+            Ok(anchor) => match anchor.anchor_text {
+                Some(text) => Ok(text),
+                None => Err(WasmError::Serialize(SerializedBytesError::Deserialize(
+                    "missing anchor text".into(),
+                ))),
             },
             Err(e) => Err(e),
         })
@@ -169,18 +156,34 @@ pub fn list_anchor_tags(anchor_type: String) -> ExternResult<Vec<String>> {
 #[cfg(test)]
 #[test]
 fn hash_path_root() {
-    assert_eq!(ROOT, "hdkanchor");
+    assert_eq!(ROOT, &[0_u8, 0]);
 }
 
 #[cfg(test)]
 #[test]
 fn hash_path_anchor_path() {
-    for (atype, text, path_string) in vec![
-        ("foo", None, "hdkanchor.foo"),
-        ("foo", Some("bar".to_string()), "hdkanchor.foo.bar"),
-    ] {
+    let examples = [
+        (
+            "foo",
+            None,
+            Path::from(vec![
+                Component::from(vec![0, 0]),
+                Component::from(vec![102, 111, 111]),
+            ]),
+        ),
+        (
+            "foo",
+            Some("bar".to_string()),
+            Path::from(vec![
+                Component::from(vec![0, 0]),
+                Component::from(vec![102, 111, 111]),
+                Component::from(vec![98, 97, 114]),
+            ]),
+        ),
+    ];
+    for (atype, text, path) in examples {
         assert_eq!(
-            Path::from(path_string),
+            path,
             (&Anchor {
                 anchor_type: atype.to_string(),
                 anchor_text: text,
@@ -193,27 +196,27 @@ fn hash_path_anchor_path() {
 #[cfg(test)]
 #[test]
 fn hash_path_anchor_entry_def() {
-    assert_eq!(Path::entry_def_id(), Anchor::entry_def_id(),);
+    assert_eq!(PathEntry::entry_def_id(), Anchor::entry_def_id(),);
 
-    assert_eq!(Path::crdt_type(), Anchor::crdt_type(),);
+    assert_eq!(PathEntry::crdt_type(), Anchor::crdt_type(),);
 
-    assert_eq!(Path::required_validations(), Anchor::required_validations(),);
+    assert_eq!(
+        PathEntry::required_validations(),
+        Anchor::required_validations(),
+    );
 
-    assert_eq!(Path::entry_visibility(), Anchor::entry_visibility(),);
+    assert_eq!(PathEntry::entry_visibility(), Anchor::entry_visibility(),);
 
-    assert_eq!(Path::entry_def(), Anchor::entry_def(),);
+    assert_eq!(PathEntry::entry_def(), Anchor::entry_def(),);
 }
 
 #[cfg(test)]
 #[test]
 fn hash_path_anchor_from_path() {
     let path = Path::from(vec![
-        Component::from(vec![
-            104, 0, 0, 0, 100, 0, 0, 0, 107, 0, 0, 0, 97, 0, 0, 0, 110, 0, 0, 0, 99, 0, 0, 0, 104,
-            0, 0, 0, 111, 0, 0, 0, 114, 0, 0, 0,
-        ]),
-        Component::from(vec![102, 0, 0, 0, 111, 0, 0, 0, 111, 0, 0, 0]),
-        Component::from(vec![98, 0, 0, 0, 97, 0, 0, 0, 114, 0, 0, 0]),
+        Component::from(vec![0, 0]),
+        Component::from(vec![102, 111, 111]),
+        Component::from(vec![98, 97, 114]),
     ]);
 
     assert_eq!(
